#ifndef _FROG_ELEMENT_EVENT_RECOCANDIDATE_H__
#define _FROG_ELEMENT_EVENT_RECOCANDIDATE_H__

#include "FROG_Element_Base.h"

class FROG_Element_Event_Candidate : public FROG_Element_Base {
public :
	int PdgId_; float P_; float Eta_; float Phi_; 

    FROG_Objects_Extended*  frogObjects_;        
    unsigned int nameId_;
	
	virtual bool isCompactible(){ return true; }

	static unsigned int sizeOf(){ return FROG_Element_Base::sizeOf() + 3*sizeof(float) + 1*sizeof(int);}

        FROG_Element_Event_Candidate() : FROG_Element_Base(C_RECO_CANDIDATE) {}

	FROG_Element_Event_Candidate(int PdgId, float P, float Eta, float Phi ) :
	FROG_Element_Base(C_RECO_CANDIDATE),	
	PdgId_(PdgId), P_(P), Eta_(Eta),Phi_(Phi)
	{    
		size_ = sizeOf();
	}

	FROG_Element_Event_Candidate(FILE* pFile) : FROG_Element_Base(C_RECO_CANDIDATE)
	{   
		size_ = sizeOf();
		fread(&PdgId_        ,sizeof(PdgId_)      ,1,pFile);
		fread(&P_            ,sizeof(P_)          ,1,pFile);
        fread(&Eta_          ,sizeof(Eta_)        ,1,pFile);
        fread(&Phi_          ,sizeof(Phi_)        ,1,pFile);		
	}

	virtual void write () {
		size_ = sizeOf();
		data_ = new unsigned char[size_-6];
		data_ = FillBuffer( data_, &PdgId_,             sizeof(PdgId_));
		data_ = FillBuffer( data_, &P_,                 sizeof(P_));
        data_ = FillBuffer( data_, &Eta_,               sizeof(Eta_));
        data_ = FillBuffer( data_, &Phi_,               sizeof(Phi_));			
		data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );		
	}

        virtual void printInfos(char* buffer){
                sprintf(buffer,"Candidate : PdgId=%+6i P=%8.2f Eta=%+6.2f Phi=%5.2f", PdgId_, P_, Eta_, Phi_);
        }

	#ifdef FROG_OPENGL
	virtual void display(bool UseDisplayList=true, float* color=NULL){
		// first Init Colors & Style!
		init(NULL);
		if(P_<style_->EMin_){return;}
		if(FROG_COORD::GetPt(P_,Eta_)<style_->PtMin_){return;}
		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}

		if(Start_Display(UseDisplayList)){
			glLoadName( nameId_ );

			float L1      = log(1+P_)*15;			
			float L2      = L1*0.8;
			//float Radius1 = 0.2;
			float Radius1 = L1*0.05;
			float Radius2 = Radius1 / 2;

			glPushMatrix();
			glRotatef(57.29*Phi_                        ,0,0,1);
			glRotatef(57.29*FROG_COORD::EtaToTheta(Eta_),0,1,0);		

			float dphi = 0.62832;
			glBegin (GL_QUADS); 			
				for(double phi=0; phi<=6.2831;phi+=dphi){
					glNormal3f(cos(phi),sin(phi),0);
					glVertex3f (Radius2*cos(phi)	  ,Radius2*sin(phi)		 , 0);
					glVertex3f (Radius2*cos(phi+dphi) ,Radius2*sin(phi+dphi) , 0);
					glVertex3f (Radius2*cos(phi+dphi) ,Radius2*sin(phi+dphi) , L2);
					glVertex3f (Radius2*cos(phi)	  ,Radius2*sin(phi)		 , L2);
					
				}
			glEnd(); 
			glBegin (GL_TRIANGLES); 			
				for(double phi=0; phi<=6.2831;phi+=dphi){
					SetNormal(  0					   ,0					  ,L1,
						        Radius1*cos(phi)       ,Radius1*sin(phi)	  ,L2,
								Radius1*cos(phi+dphi)  ,Radius1*sin(phi+dphi) ,L2);
								
					glVertex3f (0                     ,0	                 , L1); 								
					glVertex3f (Radius1*cos(phi)	  ,Radius1*sin(phi)		 , L2); 
					glVertex3f (Radius1*cos(phi+dphi) ,Radius1*sin(phi+dphi) , L2); 
				}
				for(double phi=0; phi<=6.2831;phi+=dphi){
					SetNormal(  0					   ,0					  ,L2,
								Radius1*cos(phi+dphi)  ,Radius1*sin(phi+dphi) ,L2,
						        Radius1*cos(phi)       ,Radius1*sin(phi)	  ,L2);

					glVertex3f (0                     ,0	                 , L2);
					glVertex3f (Radius1*cos(phi+dphi) ,Radius1*sin(phi+dphi) , L2);
					glVertex3f (Radius1*cos(phi)	  ,Radius1*sin(phi)		 , L2);				
				}
			glEnd(); 
			glPopMatrix();

			End_Display(UseDisplayList);
		}
	}
	#endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL){	style_ = new FROG_Objects_Style(mother_->style_);
			}else{					style_ = new FROG_Objects_Style();				}	

	 	    if(PdgId_<0)frogObjects_->frogCard_->GetColor(style_->color_,"GenParticle_PdgId_%i_Color", abs(PdgId_));
				        frogObjects_->frogCard_->GetColor(style_->color_,"GenParticle_PdgId_%i_Color", PdgId_);
		}		
	}
};
#endif
