#ifndef _FROG_ELEMENT_EVENT_JET_H__
#define _FROG_ELEMENT_EVENT_JET_H__

#include "FROG_Element_Base.h"
#include "FROG_Objects_Extended.h"
#include "FROG_Coord.h"

class FROG_Element_Event_Jet : public FROG_Element_Base {
public :
    float P_; float Eta_; float Phi_; 
	unsigned int			  Components_CollEvtId_;
	std::vector<unsigned int> Components_Vector_;

	FROG_Objects_Extended*  frogObjects_;	
	unsigned int nameId_;
	unsigned int JetIndex_;

    virtual bool isCompactible(){ return false; }

    static unsigned int sizeOf(){ return FROG_Element_Base::sizeOf() + 3*sizeof(float); }

    FROG_Element_Event_Jet(float P, float Eta, float Phi):
    FROG_Element_Base(C_RECO_JET),
    P_(P), Eta_(Eta),Phi_(Phi){
            size_ = sizeOf();			
    }

    FROG_Element_Event_Jet(FILE* pFile) : FROG_Element_Base(C_RECO_JET)
    {
            size_ = sizeOf();            
			fread(&P_            ,sizeof(P_)          ,1,pFile);
            fread(&Eta_          ,sizeof(Eta_)        ,1,pFile);
            fread(&Phi_          ,sizeof(Phi_)        ,1,pFile);			
    }

    unsigned int ReadComponents(FILE* pFile, unsigned int ToRead)
    {
			if(ToRead==0)return 0;			

			fread(&Components_CollEvtId_      ,sizeof(unsigned int)     ,1,pFile);
			unsigned int NComp = (ToRead/sizeof(unsigned int))-1;
			
			unsigned int TempComp;
			for(unsigned int i=0;i<NComp;i++){
				fread(&TempComp      ,sizeof(unsigned int)     ,1,pFile);
				Components_Vector_.push_back(TempComp);			
			}

			return (NComp+1)*sizeof(unsigned int);
    }


    virtual void write () {
            size_ = sizeOf();
			if(Components_Vector_.size()>0)size_ += sizeof(unsigned int) + Components_Vector_.size()*sizeof(unsigned int);
            data_ = new unsigned char[size_-6];            
			data_ = FillBuffer( data_, &P_,                 sizeof(P_));
            data_ = FillBuffer( data_, &Eta_,               sizeof(Eta_));
            data_ = FillBuffer( data_, &Phi_,               sizeof(Phi_));			

			if(Components_Vector_.size()){
				data_ = FillBuffer( data_, &Components_CollEvtId_,              sizeof(Components_CollEvtId_));
				for(unsigned int i=0;i<Components_Vector_.size();i++){
					data_ = FillBuffer( data_, &Components_Vector_[i],          sizeof(Components_Vector_[i]));				
				}
			}

            data_ = (unsigned char*)((unsigned long)data_ - (size_-6) );
    }

	virtual void printInfos(char* buffer){		
		sprintf(buffer,"Jet : P=%7.2f Eta=%6.2f Phi=%6.2f", P_, Eta_, Phi_);
	}

    #ifdef FROG_OPENGL	
    virtual void display(bool UseDisplayList=true, float* color=NULL){	
		// first Init Colors & Style!		
		init(NULL);		
		if(P_<style_->EMin_){return;}
		if(FROG_COORD::GetPt(P_,Eta_)<style_->PtMin_){return;}

		if(color){glColor4fv(color);}else{glColor4fv(style_->color_);}
		if(color)UseDisplayList = false;
		
		if(Start_Display(UseDisplayList)){
			

			glLoadName( nameId_ );

			float L      = log(P_)*15;
			float Radius = L *0.3;

			glPushMatrix();
			glRotatef(57.29*Phi_                        ,0,0,1);
			glRotatef(57.29*FROG_COORD::EtaToTheta(Eta_),0,1,0);		
			float dphi = 0.62832;
				
			glBegin (GL_LINES); 							
				for(double phi=0; phi<=6.2831;phi+=dphi){								
					glNormal3f(cos(phi),sin(phi),0);
					glVertex3f (0                    ,0	                    , 0); 								
					glVertex3f (Radius*cos(phi)		 ,Radius*sin(phi)		, L); 
					glVertex3f (Radius*cos(phi)		 ,Radius*sin(phi)		, L); 
					glVertex3f (Radius*cos(phi+dphi) ,Radius*sin(phi+dphi)  , L); 
				}
			glEnd(); 				
			glPopMatrix();		

			if(Components_Vector_.size()>0){
				FROG_Element_Base_With_DetId* Coll = (FROG_Element_Base_With_DetId*)frogObjects_->frogEvent_->FindByDetId(Components_CollEvtId_);
				if(!Coll){End_Display(UseDisplayList);return;}

				
				for(unsigned int i=0;i<Components_Vector_.size();i++){
					FROG_Element_Base* Tower = Coll->FindByDetId_FromMap(Components_Vector_[i]);
					if(Tower){
						if(color){Tower->display(false,color);}
						else     {Tower->display(false,style_->color_);}
					}
				}				

			}

			End_Display(UseDisplayList);
		}

    }
    #endif

	virtual void init(void* frogObjects){
		if(frogObjects!=NULL){
			frogObjects_ = (FROG_Objects_Extended*) frogObjects;
			nameId_      = frogObjects_->frogEvent_->name_map.size();
			frogObjects_->frogEvent_->name_map.push_back(this);
		}else if(frogObjects==NULL && frogObjects_ != NULL){
			if(mother_ != NULL){	style_ = new FROG_Objects_Style(mother_->style_);
			}else{					style_ = new FROG_Objects_Style();				}	

			JetIndex_=1;
			float JetPt = FROG_COORD::GetPt(P_,Eta_);
			for(unsigned int i=0;i<mother_->getDaughterSize();i++){
				FROG_Element_Event_Jet* tmp = dynamic_cast<FROG_Element_Event_Jet*>(mother_->getDaughter(i));
				if(tmp==NULL || tmp==this)continue;
				if(FROG_COORD::GetPt(tmp->P_,tmp->Eta_)>JetPt)JetIndex_++;
			}			

			frogObjects_->frogCard_->GetColor(style_->color_,"Id_%i_Jet_%i_Color",((FROG_Element_Base_With_DetId*)mother_)->DetId(),JetIndex_);
		}		
	}
};

#endif
